/*
 *    GS2MGRetrieve.java
 *    Copyright (C) 2002 New Zealand Digital Library, http://www.nzdl.org
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.greenstone.gsdl3.service;


// Greenstone classes
import org.greenstone.mg.*;
import org.greenstone.gsdl3.util.*;

// XML classes
import org.w3c.dom.Element;

// General Java classes
import java.io.File;


/**
 *
 * @author <a href="mailto:kjdon@cs.waikato.ac.nz">Katherine Don</a>
 * @version $Revision: 3799 $
 */

public class GS2MGRetrieve
    extends GS2Retrieve {

    // Parameters used
    private static final String INDEX_PARAM = "index";

    // Elements used in the config file that are specific to this class
    private static final String DEFAULT_INDEX_ELEM = "defaultIndex";

    private MGWrapper mg_src_ = null;

    private String default_index_ = null;


    /** constructor */
    public GS2MGRetrieve()
    {
	System.out.println("Constructing GS2MGRetrieve...");
	mg_src_ = new MGWrapper();
    }


    /** configure this service */
    public boolean configure(Element info)
    {
	// Do specific configuration
	System.out.println("Configuring GS2MGRetrieve...");

	// Get the default index out of <defaultIndex> (buildConfig.xml)
	Element def = (Element) GSXML.getChildByTagName(info, DEFAULT_INDEX_ELEM);
	if (def != null) {
	    default_index_ = def.getAttribute(GSXML.NAME_ATT);
	}
	if (default_index_ == null || default_index_.equals("")) {
	    System.err.println("Error: default index not specified!");
	    return false;
	}
	System.out.println("Default index: " + default_index_);

	// Do generic configuration
	return super.configure(info);
    }


    /** Retrieve the structure of a document */
    protected Element processDocumentStructureRetrieve(Element request)
    {
	return super.processDocumentStructureRetrieve(request);
    }


    /** Retrieve metadata associated with a document */
    protected Element processDocumentMetadataRetrieve(Element request)
    {
	return super.processDocumentMetadataRetrieve(request);
    }


    /** Retrieve the content of a document */
    protected Element processDocumentContentRetrieve(Element request)
    {
	// Create a new (empty) result message
	Element result = doc_.createElement(GSXML.RESPONSE_ELEM);
	String from = GSPath.appendLink(cluster_name_, DOCUMENT_CONTENT_RETRIEVE_SERVICE);
	result.setAttribute(GSXML.FROM_ATT, from);
	result.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_PROCESS);
	Element result_content = doc_.createElement(GSXML.CONTENT_ELEM);
	result.appendChild(result_content);

	// Get the parameters of the request
	Element param_list = (Element) GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	if (param_list == null) {
	    System.err.println("GS2Retrieve, DocumentContentRetrieve Error: missing paramList.\n");
	    return result;  // Return the empty result
	}

	String index = default_index_;

	// Process the request parameters
	Element param = (Element) param_list.getFirstChild();
	while (param != null) {
	    if (!param.getNodeName().equals(GSXML.PARAM_ELEM)) {
		System.err.println("Warning: Non-param in paramList (ignored).");
	    }
	    else {
		//
		if (param.getAttribute(GSXML.NAME_ATT) == INDEX_PARAM) {
		    index = GSXML.getValue(param);
		    System.out.println("Index: " + index);
		}
	    }

	    param = (Element) param.getNextSibling();
	}

	// Get the request content
	Element content = (Element) GSXML.getChildByTagName(request, GSXML.CONTENT_ELEM);
	if (content == null) {
	    System.err.println("Error: DocumentContentRetrieve request had no content.\n");
	    return result;
	}

	Element doc_list = doc_.createElement(GSXML.DOCUMENT_ELEM+GSXML.LIST_MODIFIER);
	result_content.appendChild(doc_list);

	// The location of the MG index and text files
	String basedir = GSFile.collectionBaseDir(site_home_, cluster_name_)
	                 + File.separatorChar;  // Needed by MG
	String textdir = GSFile.collectionTextPath(cluster_name_);
	String indexpath = GSFile.collectionIndexPath(cluster_name_, index);
	System.out.println("Base directory: " + basedir);
	System.out.println("Text directory: " + textdir);
	System.out.println("Index path: " + indexpath);

	mg_src_.setIndex(indexpath);

	// Get the documents
	String[] doc_ids = GSXML.getDocumentNameList(content);
	for (int i = 0; i < doc_ids.length; i++) {
	    String doc_id = doc_ids[i];
	    System.out.println("Retrieving content of " + doc_id + "...");

	    long doc_num = gdbm_src_.oid2Docnum(doc_id);
	    String doc_content = mg_src_.getDocument(basedir, textdir, doc_num);

	    // For now, stick it in a text node - eventually should be parsed as xml??
	    Element doc = GSXML.createDocumentElement(doc_, doc_id);
	    GSXML.addDocText(doc_, doc, doc_content);
	    doc_list.appendChild(doc);
	}

	return result;
    }
}   
